/* CFDECRYPT: Decrypt Cold Fusion templates encrypted with CFCRYPT
   Based on original work by Matt Chapman <matthewc@cse.unsw.edu.au>

   Win32 port with file IO changes by SteelBeak <steel_beak@hotmail.com>

   Requires a DES library to link with. Tested with OpenSSL 0.9.6b
   To compile with OpenSSL using MSVC++ try the following:   
	cl cfdecrypt.c /MD /Iopenssl-0.9.6b/inc32 /link openssl-0.9.6b/out32/libeay32.lib

   Usage: cfdecrypt <input file> <output file>
*/

#include <stdio.h>
#include <stdlib.h> // for atoi() support
#include <openssl/des.h>

int usage(void)
{
	printf("Usage: CFDECRYPT <input file> <output file>\n");
	return 1;
}

int main(int argc, char **argv)
{
	FILE	*in, *out;
	char	*header = "Allaire Cold Fusion Template\012Header Size: ",
		*keystr = "Error: cannot open template file--\"%s\". Please, try again!\012\012",
		buffer[54];
	int	headsize, outlen, skip_header, len, i;

	des_cblock 		key, input, output;
	des_key_schedule	schedule;

	if (argc != 3)
		return usage();

	if((out = fopen(argv[2],"wt")) == NULL)
	{
		fprintf(stderr,"Error: Could not open output file to write.\n");
		return usage();
	}

	if((in = fopen(argv[1],"rb")) == NULL)
	{
		fclose(out);
		remove(argv[2]);
		fprintf(stderr,"Error: Could not read input file.\n");
		return usage();
	}
      

	if ((fread(buffer, 1, 54, in) < 54) || (memcmp(buffer, header, 42)))
	{
		fprintf(stderr, "File is not an encrypted template\n");
		fclose(in);
		fclose(out);
		remove(argv[2]);
		return 1;
	}

	if (!memcmp(&buffer[42], "New Version", 11))
	{
		headsize = 69;
		skip_header = 1;
	}
	else
	{
		headsize = atoi(&buffer[42]);
		skip_header = 0;
	}

	if ((headsize < 54) || (fseek(in, headsize, SEEK_SET) < 0))
	{
		fprintf(stderr, "Error in file format\n");
		fclose(in);
		fclose(out);
		remove(argv[2]);
		return 1;
	}

	des_string_to_key(keystr, &key);
	des_set_key(&key, schedule);
	outlen = 0;

	while ((len = fread(input, 1, 8, in)) == 8)
	{
		des_ecb_encrypt(&input, &output, schedule, 0);
		outlen += 8;
		i = 0;

		if (skip_header)
		{
			while (i < 8)
			{
				if (output[i++] == 0x1A)
				{
					skip_header = 0;
					break;
				}
			}
		}
		fwrite(output + i, 1, 8 - i, out);
	}

	for (i = 0; i < len; i++)
	{
		output[i] = input[i] ^ (outlen + i);
	}
	fwrite(output, 1, len, out);

	fclose(in);
	fclose(out);

	return 0;
}